using System;
using System.Data;
using System.Web.UI.WebControls;

using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.SettingsProvider;
using CMS.CMSHelper;
using CMS.UIControls;
using CMS.SiteProvider;

using TreeNode = CMS.TreeEngine.TreeNode;
using TimeZoneInfo = CMS.SiteProvider.TimeZoneInfo;

public partial class CMSModules_Content_CMSDesk_Properties_Workflow : CMSPropertiesPage
{
    #region "Private variables"

    // Current step ID
    private int currentStepId = 0;

    // Current Node
    private TreeNode mNode = null;
    private TreeProvider mTree = null;
    private WorkflowManager mWorkflowManager = null;

    private UserInfo currentUserInfo = null;
    private SiteInfo currentSiteInfo = null;
    private TimeZoneInfo usedTimeZone = null;
    private WorkflowInfo mWorkflowInfo = null;
    private bool wfInfoBinded = false;
    private bool? mCanUserApprove = null;

    #endregion


    #region "Properties"

    private TreeProvider Tree
    {
        get
        {
            return mTree ?? (mTree = new TreeProvider(CMSContext.CurrentUser));
        }
    }


    private WorkflowManager WorkflowManager
    {
        get
        {
            return mWorkflowManager ?? (mWorkflowManager = new WorkflowManager(Tree));
        }
    }


    private TreeNode Node
    {
        get
        {
            return mNode ?? (mNode = DocumentHelper.GetDocument(NodeID, CMSContext.PreferredCultureCode, Tree));
        }
    }


    private bool CanUserApprove
    {
        get
        {
            if (mCanUserApprove == null)
            {
                mCanUserApprove = WorkflowManager.CanUserApprove(Node, CMSContext.CurrentUser);
            }
            return mCanUserApprove.Value;
        }
    }


    private bool DocumentIsCheckedOut
    {
        get
        {
            return (Node != null) && (Node.GetValue("DocumentCheckedOutByUserID") != null);
        }
    }


    private static int NodeID
    {
        get
        {
            // Current Node ID        
            return QueryHelper.GetInteger("nodeid", 0);
        }
    }


    private WorkflowInfo WorkflowInfo
    {
        get
        {
            if ((mWorkflowInfo == null) && !wfInfoBinded)
            {
                mWorkflowInfo = WorkflowManager.GetNodeWorkflow(Node);
                wfInfoBinded = true;
            }
            return mWorkflowInfo;
        }
    }


    /// <summary>
    /// Determines whether to send workflow emails
    /// </summary>
    private static bool SendWorkflowEmails
    {
        get
        {
            return SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSiteName + ".CMSSendWorkflowEmails");
        }
    }

    #endregion


    #region "Page events"

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);

        if (!CMSContext.CurrentUser.IsAuthorizedPerUIElement("CMS.Content", "Properties.Workflow"))
        {
            RedirectToCMSDeskUIElementAccessDenied("CMS.Content", "Properties.Workflow");
        }

        // Hide custom steps if license doesn't allow them
        if (!WorkflowInfoProvider.IsCustomStepAllowed())
        {
            gridSteps.WhereCondition = "((StepName = 'edit') OR (StepName = 'published') OR (StepName = 'archived'))";
        }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        // Register the scripts
        ScriptHelper.RegisterProgress(Page);

        UIContext.PropertyTab = PropertyTabEnum.Workflow;
        // Turn sorting off
        gridSteps.GridView.AllowSorting = false;

        // Prepare the query parameters
        object[,] parameters = new object[1, 3];
        parameters[0, 0] = "@DocumentID";
        parameters[0, 1] = 0;

        // Prepare the steps query parameters
        object[,] stepsParameters = new object[1, 3];
        stepsParameters[0, 0] = "@StepWorkflowID";
        stepsParameters[0, 1] = 0;

        if (Node != null)
        {
            // Check read permissions
            if (CMSContext.CurrentUser.IsAuthorizedPerDocument(Node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), Node.NodeAliasPath));
            }

            // Prepare parameters
            parameters[0, 1] = Node.DocumentID;
            currentStepId = ValidationHelper.GetInteger(Node.GetValue("DocumentWorkflowStepID"), 0);

            if (WorkflowInfo != null)
            {
                stepsParameters[0, 1] = WorkflowInfo.WorkflowID;
                lblWorkflowName.Text = string.Format(ResHelper.GetString("WorfklowProperties.lblWorkflowName"), WorkflowInfo.WorkflowDisplayName);
                // Initialize unigrids
                gridHistory.OnExternalDataBound += gridHistory_OnExternalDataBound;
                gridSteps.OnExternalDataBound += gridSteps_OnExternalDataBound;
                gridHistory.ZeroRowsText = ResHelper.GetString("workflowproperties.nohistoryyet");
            }
        }
        else
        {
            pnlContent.Visible = false;
        }
        gridSteps.QueryParameters = stepsParameters;
        gridHistory.QueryParameters = parameters;
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
        ReloadData();
    }

    #endregion


    #region "Grid events"

    /// <summary>
    /// External step binding
    /// </summary>
    protected object gridSteps_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        switch (sourceName.ToLower())
        {
            case "currentstepinfo":
                DataRowView data = (DataRowView)parameter;
                if (currentStepId <= 0)
                {
                    string stepName = ValidationHelper.GetString(data["StepName"], string.Empty);
                    if (stepName.ToLower() == "edit")
                    {
                        return "<img src=\"" + GetImageUrl("CMSModules/CMS_Content/Properties/currentstep.png") + "\" alt=\"\" />";
                    }
                }
                else
                {
                    int stepId = ValidationHelper.GetInteger(data["StepID"], 0);
                    if (stepId == currentStepId)
                    {
                        return "<img src=\"" + GetImageUrl("CMSModules/CMS_Content/Properties/currentstep.png") + "\" alt=\"\" />";
                    }
                }
                return string.Empty;

            case "steporder":
                // Get grid row
                GridViewRow row = (GridViewRow)((DataControlFieldCell)sender).Parent;
                int pageOffset = (gridSteps.Pager.CurrentPage - 1) * gridSteps.Pager.CurrentPageSize;
                // Return row index
                return (pageOffset + row.RowIndex + 1).ToString();

        }
        return parameter;
    }


    /// <summary>
    /// External history binding
    /// </summary>
    protected object gridHistory_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        switch (sourceName.ToLower())
        {
            case "action":
                bool wasrejected = ValidationHelper.GetBoolean(parameter, false);
                return wasrejected ? ResHelper.GetString("WorfklowProperties.Rejected") : ResHelper.GetString("WorfklowProperties.Approved");

            case "approvedwhen":
            case "approvedwhentooltip":
                if (string.IsNullOrEmpty(parameter.ToString()))
                {
                    return string.Empty;
                }
                else
                {
                    if (currentUserInfo == null)
                    {
                        currentUserInfo = CMSContext.CurrentUser;
                    }

                    if (currentSiteInfo == null)
                    {
                        currentSiteInfo = CMSContext.CurrentSite;
                    }

                    if (sourceName == "approvedwhen")
                    {
                        return TimeZoneHelper.GetCurrentTimeZoneDateTimeString(ValidationHelper.GetDateTime(parameter, DateTimeHelper.ZERO_TIME),
                            currentUserInfo, currentSiteInfo, out usedTimeZone);
                    }
                    else
                    {
                        if (TimeZoneHelper.TimeZonesEnabled() && (usedTimeZone == null))
                        {
                            TimeZoneHelper.GetCurrentTimeZoneDateTimeString(ValidationHelper.GetDateTime(parameter, DateTimeHelper.ZERO_TIME),
                                currentUserInfo, currentSiteInfo, out usedTimeZone);
                        }
                        return TimeZoneHelper.GetGMTLongStringOffset(usedTimeZone);
                    }
                }

            case "formattedusername":
                DataRowView drv = (DataRowView)parameter;
                string userName = ValidationHelper.GetString(drv["UserName"], String.Empty);
                string fullName = ValidationHelper.GetString(drv["FullName"], String.Empty);

                return HTMLHelper.HTMLEncode(Functions.GetFormattedUserName(userName, fullName));

        }
        return parameter;
    }

    #endregion


    #region "Protected methods"

    /// <summary>
    /// Reloads the page data
    /// </summary>
    protected void ReloadData()
    {
        SetupForm();
        gridHistory.ReloadData();
        gridSteps.ReloadData();
    }


    private void ClearComment()
    {
        txtComment.Text = string.Empty;
    }


    /// <summary>
    /// Reloads the form status
    /// </summary>
    protected void SetupForm()
    {
        // Check modify permissions
        if (CMSContext.CurrentUser.IsAuthorizedPerDocument(Node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
        {
            pnlForm.Enabled = false;
            btnArchive.Visible = false;
            lblInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), Node.NodeAliasPath);
        }
        else
        {
            if (currentStepId > 0)
            {
                // Setup the form
                WorkflowStepInfo stepInfo = WorkflowStepInfoProvider.GetWorkflowStepInfo(currentStepId);
                if (stepInfo != null)
                {
                    switch (stepInfo.StepName.ToLower())
                    {
                        case "published":
                            btnApprove.Enabled = false;
                            btnApprove.Visible = false;
                            btnReject.Enabled = (Node.DocumentCheckedOutVersionHistoryID > 0) && CanUserApprove && !DocumentIsCheckedOut;
                            btnReject.Visible = true;
                            btnArchive.Enabled = CanUserApprove && !DocumentIsCheckedOut;
                            btnArchive.Visible = true;
                            break;

                        case "archived":
                            btnApprove.Enabled = false;
                            btnApprove.Visible = false;
                            btnReject.Enabled = false;
                            btnReject.Visible = false;
                            btnArchive.Enabled = false;
                            btnArchive.Visible = false;
                            pnlForm.Enabled = false;
                            break;

                        case "edit":
                            btnApprove.Enabled = !DocumentIsCheckedOut;
                            btnApprove.Visible = true;
                            btnReject.Enabled = false;
                            btnReject.Visible = false;
                            btnArchive.Enabled = false;
                            btnArchive.Visible = false;
                            break;

                        default:
                            btnApprove.Enabled = CanUserApprove && !DocumentIsCheckedOut;
                            btnApprove.Visible = true;
                            btnReject.Enabled = CanUserApprove && !DocumentIsCheckedOut;
                            btnReject.Visible = true;
                            btnArchive.Enabled = false;
                            btnArchive.Visible = false;
                            break;


                    }
                    bool actionIsAllowed = btnApprove.Enabled || btnReject.Enabled || btnArchive.Enabled;

                    pnlForm.Enabled = actionIsAllowed && !DocumentIsCheckedOut;
                    lblSendMail.Enabled = actionIsAllowed;
                    chkSendMail.Enabled = actionIsAllowed;
                    plcSendMail.Visible = SendWorkflowEmails;
                }
            }
            else
            {
                // No workflow operation has been done yet
                btnApprove.Enabled = !DocumentIsCheckedOut;
                btnApprove.Visible = true;
                btnReject.Enabled = false;
                btnReject.Visible = false;
                btnArchive.Enabled = false;
                btnArchive.Visible = false;
            }
        }

        // Check if document isn't checked out
        if (DocumentIsCheckedOut)
        {
            lblInfo.Text = ResHelper.GetString("WorfklowProperties.DocumentIsCheckedOut");
        }

        // If no workflow scope set for node, hide the data  
        if (WorkflowInfo == null)
        {
            lblInfo.Text = ResHelper.GetString("workflowproperties.scopenotset");
            pnlWorkflow.Visible = false;
        }
    }

    #endregion


    #region "Button handling"

    /// <summary>
    /// Approve event handler
    /// </summary>
    protected void btnApprove_Click(object sender, EventArgs e)
    {
        // Check modify permissions
        if (CMSContext.CurrentUser.IsAuthorizedPerDocument(Node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
        {
            return;
        }
        
        // Get original step
        WorkflowStepInfo originalStep = WorkflowManager.GetStepInfo(Node);

        // Approve document
        WorkflowStepInfo nextStep = WorkflowManager.MoveToNextStep(Node, txtComment.Text);

        // Send workflow e-mails
        if (chkSendMail.Checked && SendWorkflowEmails)
        {
            if ((nextStep == null) || (nextStep.StepName.ToLower() == "published"))
            {
                // Publish e-mails
                WorkflowManager.SendWorkflowEmails(Node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Published, txtComment.Text);
            }
            else
            {
                // Approve e-mails
                WorkflowManager.SendWorkflowEmails(Node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Approved, txtComment.Text);
            }
        }

        ClearComment();


        if (nextStep != null)
        {
            currentStepId = nextStep.StepID;
        }
        else
        {
            // If no next step (workflow cancelled), hide the form
            lblInfo.Text = ResHelper.GetString("WorfklowProperties.WorkflowFinished");
            pnlWorkflow.Visible = false;
        }
    }


    /// <summary>
    /// Reject event handler
    /// </summary>
    protected void btnReject_Click(object sender, EventArgs e)
    {
        // Check modify permissions
        if (CMSContext.CurrentUser.IsAuthorizedPerDocument(Node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
        {
            return;
        }
        
        // Get original step
        WorkflowStepInfo originalStep = WorkflowManager.GetStepInfo(Node);

        // Reject document
        WorkflowStepInfo previousStep = WorkflowManager.MoveToPreviousStep(Node, txtComment.Text);
        currentStepId = previousStep.StepID;

        // Send workflow e-mails
        if (chkSendMail.Checked && SendWorkflowEmails)
        {
            WorkflowManager.SendWorkflowEmails(Node, CMSContext.CurrentUser, originalStep, previousStep, WorkflowActionEnum.Rejected, txtComment.Text);
        }

        ClearComment();
    }


    /// <summary>
    /// Archive event handler
    /// </summary>
    protected void btnArchive_Click(object sender, EventArgs e)
    {
        // Check modify permissions
        if (CMSContext.CurrentUser.IsAuthorizedPerDocument(Node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
        {
            return;
        }
        
        // Get original step
        WorkflowStepInfo originalStep = WorkflowManager.GetStepInfo(Node);

        // Archive document
        WorkflowStepInfo nextStep = WorkflowManager.ArchiveDocument(Node, txtComment.Text);
        currentStepId = nextStep.StepID;

        // Send workflow e-mails
        if (chkSendMail.Checked && SendWorkflowEmails)
        {
            WorkflowManager.SendWorkflowEmails(Node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Archived, txtComment.Text);
        }

        ClearComment();

        // Refresh tree
        ScriptHelper.RegisterStartupScript(this, typeof(string), "refreshTree", ScriptHelper.GetScript("RefreshTree(" + Node.NodeParentID + ", " + Node.NodeID + ");"));
    }

    #endregion
}
